"""Medium API.
"""

import requests

from datahunters.social.common import SocialUser


class MediumAPI(object):
  BASE_URL = "https://api.medium.com/v1"

  def __init__(self, client_id, client_secret):
    pass



"""Test uploading image file to s3 and get url.
"""

import json

import boto3
from boto3.s3.transfer import S3Transfer

info = json.load(open("./aws_db_config.json", "r"))

credentials = {
    'aws_access_key_id': "AKIAILIZOLWO7H6IZVPA",
    'aws_secret_access_key': "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"
}

client = boto3.client('s3', 'us-east-1', **credentials)

# set content type.
# http://s3browser.com/features-content-mime-types-editor.aspx

# upload.
client.upload_file(
    './11199831_5723599_1000.jpg',
    'eyestyle.random',
    'test.jpg',
    extra_args={'ACL': 'public-read',
                'ContentType': 'image/jpeg'})

with open("./11199831_5723599_1000.jpg", "rb") as f:
  client.upload_fileobj(
      f,
      "eyestyle.random",
      "test.jpg",
      extra_args={'ACL': 'public-read',
                  'ContentType': 'image/jpeg'})

file_url = '%s/%s/%s' % (client.meta.endpoint_url, 'eyestyle.random',
                         'test.jpg')
print file_url

# download.
client.download_file("eyestyle.random", "test.jpg", "./test.jpg")
with open("./test.jpg", "wb") as f:
  client.download_fileobj("eyestyle.random", "test.jpg", f)



"""Compute summary of data in database.
"""

import json
import pymongo


def run_summary():
  """Compute statistics of the database.
  """
  # parse config.
  configs = json.load(open("config.json", "r"))
  # make connection.
  db_client = pymongo.MongoClient("mongodb://localhost:27017")
  database = db_client[configs["db_name"]]
  collection = database[configs["collection_name"]]
  collection.create_index("category")
  # get a list of categories.
  categories = collection.distinct("category")
  print categories
  # compute product number for each category.
  product_stats = {}
  for cate in categories:
    cate_count = collection.find({"category": cate}).count()
    product_stats[cate] = cate_count
    print "{}: {}".format(cate, cate_count)


if __name__ == "__main__":
  run_summary()



"""Common data structure shared across modules.
"""

import abc

import json


class DataObjectBase(object):
  """Base class for data object.
  """
  __metaclass__ = abc.ABCMeta

  def to_json(self):
    """Convert data members to json object.
    """
    return json.loads(json.dumps(self.__dict__))


if __name__ == "__main__":
  eg = DataObjectBase()
  print(eg.__dict__)



"""arXiv api.

https://arxiv.org/help/api
"""

import requests
import atoma


class arXivAPI(object):
  """Class of arXiv api.
  """

  def __init__(self):
    self.API_URL = "http://export.arxiv.org/api"

  def search(self, title_query, field, num_results=10):
    """Search for publications.

    Args:
      num_results: number of items to return. get all by setting -1.
    """
    items = []
    for start_id in range(0, num_results, 10):
      max_results = 10 if num_results - start_id >= 10 else num_results - start_id
      req_url = "{}/query?search_query=ti:{}&cat={}&start={}&max_results={}".format(
          self.API_URL, title_query, field, start_id, max_results)
      res = requests.get(req_url)
      feed = atoma.parse_atom_bytes(res.content)
      # parse results into items.
      for entry in feed.entries:
        item = {}
        item["title"] = entry.title.value
        item["authors"] = [author.name for author in entry.authors]
        item["links"] = [(link.href, link.title) for link in entry.links]
        item["publish_date"] = entry.published.isoformat()
        item["update_date"] = entry.updated.isoformat()
        item["summary"] = entry.summary.value
        items.append(item)
    return items


if __name__ == "__main__":
  # quick test.
  api = arXivAPI()
  items = api.search("dataset", "cs.CV", 23)
  # print(items)
  assert len(items) == 23



"""Tests for google image search.
"""

from datahunters.search.google.api import GoogleImageAPI


class TestGoogleAPI(object):
  """Test class.
  """

  def test_simple_scrape(self):
    """Test scraping default page.
    """
    engine = GoogleImageAPI()
    all_imgs = engine.scrape("cats", False)
    assert len(all_imgs) > 0

  def test_inf_scroll_scrape(self):
    """Test full scrape with infinite scrolling.
    """
    engine = GoogleImageAPI()
    all_imgs = engine.scrape("cats", True)
    assert len(all_imgs) > 0



"""Robinhood api.

https://github.com/sanko/Robinhood
"""

import json
import requests


# TODO: api not working.
class RobinhoodAPI(object):
  """Class for Robinhood API (unofficial).
  """

  def __init__(self, username=None, password=None):
    self.token = None
    if username is None or password is None:
      print(
          "You need to provide username and password to authenticate. Sign up an account on Robinhood."
      )
    else:
      return
      req_url = "https://api.robinhood.com/oauth2/token/"
      res = requests.post(req_url, {
          "username": username,
          "password": password
      })
      res.raise_for_status()
      self.token = res.json()["token"]

  def get_fundamentals(self, company_symbols):
    """Get fundemantal data of given companies.
    """
    req_url = "https://api.robinhood.com/fundamentals/?symbols="
    for symbol in company_symbols:
      req_url += symbol + ","
    # remove last comma
    req_url = req_url[:-2]
    res = requests.get(req_url)
    res.raise_for_status()
    print(res.json())


if __name__ == "__main__":
  api = RobinhoodAPI("flyfengjie@gmail.com", "detective3!")
  api.get_fundamentals(["MSFT", "FB", "TSLA"])



"""Crawl instagram data and save to aws.
"""

import datetime
import json
import os
import time

from tqdm import tqdm
import concurrent.futures

import owl
from owl.data import img_tools
from owl.third.aws import aws

import instagram_scraper

logger_name = "instagram_crawler_logger"
logger = owl.data.common.get_logger(
    logger_name, os.path.join("extra", logger_name + ".log"))


class InstagramCrawlerAWS(instagram_scraper.InstagramScraper):
  """Class for crawling instagram data.
  """
  table_name = "data___instagram"

  def __init__(self):
    instagram_scraper.InstagramScraper.__init__(
        self,
        media_metadata=True,
        media_types=[],
        include_location=True,
        comments=False,
        destination="./",
        maximum=0)

  def init(self):
    """Init db.
    """
    try:
      self.db_config = json.load(open("./configs/aws_db_config.json", "r"))
      self.aws_man = aws.AWSIO(
          access_key=self.db_config["access_key"],
          secret_key=self.db_config["secret_access_key"],
          region=self.db_config["region"])
      self.aws_man.s3_man.use_bucket(bucket_name=self.db_config["s3_bucket"])
      logger.info("using s3 bucket: {}".format(self.db_config["s3_bucket"]))
      self.aws_man.dynamodb_man.create_table(self.table_name, "item_id", None,
                                             10, 10)
    except Exception as ex:
      logger.error("init db error: {}".format(ex.message))
      raise

  def crawl_user(self, username, last_time_stamp=None):
    """Scrape media for a user and return json results.
    """
    logger.info("start crawling {}".format(username))

    executor = concurrent.futures.ThreadPoolExecutor(max_workers=1)

    if last_time_stamp is not None:
      self.last_scraped_filemtime = last_time_stamp
    self.posts = []
    self.last_scraped_filemtime = 0
    future_to_item = {}

    # Get the user metadata.
    user = self.fetch_user(username)
    logger.info("fetched user")

    if user:
      self.get_profile_pic(None, executor, future_to_item, user, username)
      self.get_stories(None, executor, future_to_item, user, username)
    logger.info("processed user.")

    # crawl metadata.
    if self.include_location:
      media_exec = concurrent.futures.ThreadPoolExecutor(max_workers=1)

    iter_num = 0
    for item in tqdm(
        self.media_gen(username),
        desc='Searching {0} for posts'.format(username),
        unit=' media',
        disable=self.quiet):
      try:
        if self.include_location:
          media_exec.submit(self._InstagramScraper__get_location, item)

        if self.comments:
          item['comments']['data'] = list(
              self.query_comments_gen(item['code']))

        if self.media_metadata or self.comments or self.include_location:
          self.posts.append(item)

        # save item to aws.
        self.save_item(item)

        iter_num = iter_num + 1
        if self.maximum != 0 and iter_num >= self.maximum:
          break
      except Exception as ex:
        logger.error("error fetching item: {}".format(ex.message))

    self.logout()
    # self.save_json(self.posts, '{0}/{1}.json'.format(dst, username))
    logger.info("finish crawling {}".format(username))

  def save_item(self, item):
    """Save a given item to aws.
    """
    # check if already exists.
    res = self.aws_man.dynamodb_man.get_item({"item_id": item["id"]})
    if res is not None:
      return

    save_date = datetime.datetime.now().isoformat()
    if item["type"] != "image":
      return
    # construct object.
    es_item = {}
    es_item["item_id"] = item["id"]
    es_item["crawl_date"] = save_date
    es_item["account"] = item["user"]["username"]
    es_item["original_id"] = item["id"]
    es_item["link"] = item["link"]
    if item["location"] and "slug" in item["location"]:
      es_item["location"] = item["location"]["slug"]
    if "tags" in item:
      es_item["tags"] = item["tags"]
    es_item["likes"] = item["likes"]["count"]
    if item["caption"] and "text" in item["caption"]:
      es_item["caption"] = item["caption"]["text"]
    es_item["raw_metadata"] = json.dumps(item)
    img_info = item["images"]["standard_resolution"]
    es_item["img_url"] = img_info["url"]
    es_item["img_width"] = img_info["width"]
    es_item["img_height"] = img_info["height"]

    # upload the image data to aws.
    img_url = item["urls"][0]
    # download image.
    startt = time.time()
    img_bin, img_ext = img_tools.download_img_from_url(img_url)
    print "download image time: {}s".format(time.time() - startt)
    saved_img_url = self.aws_man.s3_man.upload_img_bin_as_file(
        self.db_config["s3_bucket"], img_bin, img_ext)
    print "save image time: {}s".format(time.time() - startt)
    es_item["raw_img_url"] = saved_img_url
    es_item["img_format"] = img_ext

    # add to db.
    startt = time.time()
    self.aws_man.dynamodb_man.add_item(es_item)
    print "save dynamodb time: {}s".format(time.time() - startt)


def run():
  crawler = InstagramCrawlerAWS()
  crawler.init()

  with open("./configs/targets.json", "r") as f:
    metadata = json.load(f)
  usernames = metadata["users"]
  for username in usernames:
    crawler.crawl_user(username)


if __name__ == "__main__":
  run()



"""Google api class.
"""

import json

from datahunters.shared.selenium_scraper import SeleniumScraper
from datahunters.search.common import ImgSearchObject


class BingImageAPI(SeleniumScraper):
  """Class for google image search api.
  """
  base_url = "https://www.bing.com/images/search"

  def __init__(self, use_headless=True):
    super().__init__(use_headless)
    print("Bing image api initialized")

  def scrape(self, keywords, get_all=False):
    all_res = []
    try:
      print("start scraping {} using Bing".format(keywords))
      formatted_keywords = keywords.strip().replace(" ", "+")
      req_url = "{}?q={}&qs=n&form=QBILPG&sp=-1&sc=8-3&sk=&cvid=1B063871C36E42438CC99549B92CD76D".format(
          self.base_url, formatted_keywords)
      if get_all:
        elems = self.scrape_inf_scroll(
            req_url, "a.iusc", "a.iusc", load_btn_selector="a.btn_seemore")
      else:
        self.load_content(req_url)
        elems = self.find_elements("a.iusc")
      print("total fetched items: {}".format(len(elems)))
      for elem in elems:
        try:
          # create result object.
          cur_res = ImgSearchObject()
          raw_data = self.get_attribute_values([elem], "m")[0]
          # parse link.
          parsed_data = json.loads(raw_data)
          cur_res.img_url = parsed_data["murl"]
          all_res.append(cur_res)
        except Exception as ex:
          print("error in processing item: {}".format(ex))
          continue
      print("Bing image scraping finished.")
      return all_res
    except Exception as ex:
      print("error in Bing image scraper: {}".format(ex))
      return all_res


if __name__ == "__main__":
  api = BingImageAPI()
  # warm up
  all_imgs = api.scrape("cats", False)
  all_imgs = api.scrape("dental xray", True)
  print("scrape finished.")



"""Scraper for instagram.
"""

import datetime
import json
import requests
from tqdm import tqdm
import concurrent.futures


class InstagramUser(object):
  """User class.
  """

  def __init__(self):
    self.user_id = ""
    self.username = ""
    self.fullname = ""
    self.profile_img_url = ""
    self.num_posts = 0
    self.num_followers = 0
    self.num_following = 0
    self.bio = ""
    self.is_verified = False
    self.is_private = False

  def to_json(self):
    return json.loads(json.dumps(self.__dict__))


class InstagramPost(object):
  """Class of an Instagram post.
  """

  def __init__(self):
    self.post_id = ""
    self.account_id = ""
    self.caption = ""
    self.post_date = ""
    self.num_likes = 0
    self.num_comments = 0
    self.img_url = ""
    self.img_width = 0
    self.img_height = 0
    self.vid_url = ""
    self.comments = []
    self.tags = []
    self.location = None

  def __str__(self):
    return "post: {}, published at: {}, img: {}, h: {}, w: {}".format(
        self.post_id, self.post_date, self.img_url, self.img_width,
        self.img_height)

  def to_json(self):
    return json.loads(json.dumps(self.__dict__))


class InstagramAPI(object):
  """Class for Instagram data.
  """

  def __init__(self):
    self.engine = None
    self.base_url = "https://www.instagram.com/"

  def convert_raw_post(self, raw_post):
    """Converting raw post json to object.

    The post json corresponds to edge node.
    """
    post = InstagramPost()
    post.account_id = raw_post["owner"]["id"]
    post.img_url = raw_post["display_url"]
    post.img_width = raw_post["dimensions"]["width"]
    post.img_height = raw_post["dimensions"]["height"]
    post.num_likes = raw_post["edge_media_preview_like"]["count"]
    post.num_comments = raw_post["edge_media_to_comment"]["count"]
    post.post_date = datetime.datetime.fromtimestamp(
        int(raw_post["taken_at_timestamp"])).isoformat()
    post.post_id = raw_post["id"]
    post.caption = raw_post["edge_media_to_caption"]["edges"][0]["node"][
        "text"]
    return post

  def get_user_posts(self, username, only_img=True, start_stamp=""):
    """Fetch user post.

    Returns:
      a generator for user post.
    """
    user_info = self.get_user_info(username)
    end_cursor = start_stamp
    while True:
      # request one page.
      MEDIA_URL = "{}graphql/query/?query_id=17888483320059182&id={}&first=100&after={}".format(
          self.base_url, user_info.user_id, end_cursor)
      res = requests.get(MEDIA_URL).json()
      if res["status"] == "ok":
        res_data = res["data"]["user"]["edge_owner_to_timeline_media"]
        page_info = res_data["page_info"]
        post_data = res_data["edges"]
        for raw_post in post_data:
          raw_post = raw_post["node"]
          if only_img and raw_post["is_video"]:
            continue
          cur_post = self.convert_raw_post(raw_post)
          yield cur_post
        if page_info["has_next_page"]:
          end_cursor = page_info["end_cursor"]
        else:
          break
      else:
        break

  def get_user_info(self, username):
    """Get information about a user.
    """
    USER_URL = "{}{}/?__a=1".format(self.base_url, username)
    res = requests.get(USER_URL).json()
    user_info = res["user"]
    cur_user = InstagramUser()
    cur_user.user_id = user_info["id"]
    cur_user.username = user_info["username"]
    cur_user.fullname = user_info["full_name"]
    cur_user.bio = user_info["biography"]
    cur_user.is_private = user_info["is_private"]
    cur_user.is_verified = user_info["is_verified"]
    cur_user.num_following = user_info["follows"]["count"]
    cur_user.num_followers = user_info["followed_by"]["count"]
    cur_user.profile_img_url = user_info["profile_pic_url_hd"]
    cur_user.num_posts = user_info["media"]["count"]
    return cur_user


if __name__ == "__main__":
  scraper = InstagramAPI()
  media = scraper.get_user_posts("livecandid")
  print next(media)
  print next(media)



"""Crawler for amazon products.
"""

import amazonproduct

class AmazonParams(object):
  """Parameters for crawling amazon.
  """
  pass

class AmazonCrawler(object):
  """Crawler class for amazon products.
  """
  def __init__(self):
    pass


"""API for unplash.

https://unsplash.com/developers
"""

import json
import objectpath
import requests

from datahunters.stock_img.common import StockContent, StockImageAPIBase


class UnsplashImage(StockContent):
  """Class for unsplash image.
  """
  collection = ""
  likes = 0

  def __str__(self):
    return "unsplash image: {}".format(self.full_url)


class UnsplashAPI(StockImageAPIBase):
  """Class for Unsplash API.
  """
  base_url = "https://api.unsplash.com"

  def __init__(self, api_key):
    self.api_key = api_key

  def convert_raw_photo(self, raw_photo):
    """Convert raw photo json to stockimage object.
    """
    stock_img = UnsplashImage()
    stock_img.img_id = raw_photo["id"]
    stock_img.created_date = raw_photo["created_at"]
    stock_img.description = raw_photo["description"]
    stock_img.full_url = objectpath.Tree(raw_photo).execute("$.urls.full")
    stock_img.normal_url = objectpath.Tree(raw_photo).execute("$.urls.regular")
    stock_img.link = objectpath.Tree(raw_photo).execute("$.links.html")
    stock_img.owner = objectpath.Tree(raw_photo).execute("$.user.username")
    stock_img.likes = raw_photo["likes"]
    return stock_img

  def parse_img_res_headers(self, headers):
    """Extract info from response headers.
    """
    total_page_num = None
    next_page_link = None
    remain_limits = headers["X-Ratelimit-Remaining"]
    if remain_limits == "0":
      return None, None, 0
    links = headers["Link"]
    link_parts = links.split(",")
    for link_part in link_parts:
      if link_part.find("last") != -1:
        page_info = link_part.split("&")[-2]
        total_page_num = int(page_info.split("=")[-1])
      if link_part.find("next") != -1:
        next_page_link = link_part.split(";")[0][1:-1]
    next_page_link = next_page_link.strip("<")
    next_page_link = next_page_link.strip(">")
    return total_page_num, next_page_link, remain_limits

  def get_imgs(self, start_id=0, item_num=30):
    """Retrieve images from a start page with number.

    Args:
      start_id: start item id.
      item_num: number of items to fetch.

    Returns:
      list of image objects.
    """
    print("start fetching images...")
    num_per_page = 30
    # compute start page id.
    start_page = start_id // num_per_page + 1
    # item id on starting page.
    page_item_id = start_id % num_per_page
    next_page_link = "{}/photos?client_id={}&page={}&per_page={}".format(
        self.base_url, self.api_key, start_page, num_per_page)
    all_res = []
    while next_page_link:
      try:
        # get current page results.
        res = requests.get(next_page_link)
        # get next page link.
        _, next_page_link, remain_limits = self.parse_img_res_headers(
            res.headers)
        if remain_limits == 0:
          print("request limits exceeded.")
          break
        res_json = res.json()
        for raw_photo in res_json[page_item_id:]:
          cur_img_obj = self.convert_raw_photo(raw_photo)
          all_res.append(cur_img_obj)
          if len(all_res) == item_num:
            break
        if len(all_res) >= item_num:
          break
        # reset starting item id.
        page_item_id = 0
      except Exception as ex:
        print("error processing get_imgs request {}: {}".format(
            next_page_link, ex))
        break
    return all_res

  def get_random_imgs(self):
    """Fetch random images.
    """
    all_imgs = []
    next_link = "{}/photos/random?client_id={}&count=30".format(
        self.base_url, self.api_key)
    while True:
      res = requests.get(next_link)
      res_json = res.json()
      for raw_photo in res_json:
        cur_img_obj = self.convert_raw_photo(raw_photo)
        all_imgs.append(cur_img_obj)
      break
    return all_imgs

  def search_imgs(self, keywords):
    """Search images with keywords.
    """
    next_page_link = "{}/search/photos?client_id={}&query={}&page=1&per_page=30".format(
        self.base_url, self.api_key, keywords)
    all_imgs = []
    # while next_page_link:
    res = requests.get(next_page_link)
    # get images.
    res_json = res.json()
    for raw_photo in res_json["results"]:
      cur_img_obj = self.convert_raw_photo(raw_photo)
      all_imgs.append(cur_img_obj)
    # get link to next page.
    _, next_page_link, _ = self.parse_img_res_headers(res.headers)
    return all_imgs



"""Crawler for amazon products.
"""

import amazonproduct

class AmazonCrawler(object):
    """
    """


amz_api = amazonproduct.API(
    access_key_id='AKIAIPBHBNDZV3LGYH5A',
    secret_access_key='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',
    associate_tag='eyes0ac-20',
    locale='us')

products = amz_api.item_search(
    search_index='Fashion', Keywords='dress', ResponseGroup='Large, Images')
print 'total products: {}'.format(len(products))
for product in products:
  print '{}'.format(product.ItemAttributes.Title)



"""For distribution.
"""

from setuptools import setup, find_packages, find_namespace_packages
from setuptools.command.develop import develop
from setuptools.command.install import install
from subprocess import check_call


class PostDevelopCommand(develop):
  """Post-installation for development mode."""

  def run(self):
    check_call("apt-get install this-package".split())
    develop.run(self)


class PostInstallCommand(install):
  """Post-installation for installation mode."""

  def run(self):
    # PUT YOUR POST-INSTALL SCRIPT HERE or CALL A FUNCTION
    install.run(self)


setup(
    name="datahunters",
    version="0.0.9",
    description="library for collecting data",
    url="https://github.com/perceptance/datahunters",
    author="Jie Feng",
    author_email="jiefeng@perceptance.io",
    license="MIT",
    include_package_data=True,
    package_dir={"": "src"},
    packages=find_namespace_packages(where="src"),
    install_requires=[],
    test_require=[])



"""API class for adobe stock content.

https://www.adobe.io/apis/creativecloud/stock/docs/getting-started/01-getting-started_master.html
"""

import json
import objectpath
import requests

from datahunters.stock_img.common import StockContent, StockImageAPIBase


class AdobeStockContent(StockContent):
  """Class for adobe stock image.
  """
  category_name = ""


class AdobeStockAPI(StockImageAPIBase):
  """Class for adobe stock api.
  """
  client_id = ""
  client_secret = ""
  headers = {}
  base_url = "https://stock.adobe.io/Rest/"

  def __init__(self, client_id_):
    """Set up client keys.
    """
    self.client_id = client_id_
    self.headers = {
        "X-Product": "MySampleApp/1.0",
        "X-API-Key": self.client_id
    }

  def convert_raw_content(self, raw_content):
    """Convert raw content json to object.
    """
    cur_item = AdobeStockContent()
    cur_item.img_id = str(raw_content["id"])
    cur_item.description = raw_content["description"]
    if not cur_item.description:
      cur_item.description = ""
    cur_item.created_date = raw_content["creation_date"]
    cur_item.full_url = raw_content["comp_url"]
    cur_item.normal_url = raw_content["thumbnail_url"]
    cur_item.link = raw_content["details_url"]
    cur_item.title = raw_content["title"]
    cur_item.category_name = objectpath.Tree(raw_content).execute(
        "$.category.name")
    return cur_item

  def form_result_req_str(self, field_names):
    """Form total string for requested fields.
    """
    total_str = ""
    for field_name in field_names:
      total_str = "{}&result_columns[]={}".format(total_str, field_name)
    # always start with '&'.
    return total_str

  def get_imgs(self, start_page=-1):
    """Get images from the whole catalog.
    """
    if start_page == -1:
      start_page = 0
    return self.search_imgs("", start_page)

  def search_imgs(self, keywords, start_page=0):
    """Search adobe stock images.
    """
    items_per_page = 64
    offset = items_per_page * start_page
    total_num = -1
    req_fields = [
        "nb_results", "id", "stock_id", "title", "width", "height",
        "thumbnail_url", "category", "creation_date", "comp_url",
        "content_type", "details_url", "description"
    ]
    while True:
      try:
        # TODO(jiefeng): add detailed results via result column.
        req_url = "{}Media/1/Search/Files?locale=en_US&search_parameters[limit]=64&search_parameters[words]={}&search_parameters[offset]={}".format(
            self.base_url, keywords, offset)
        req_url += self.form_result_req_str(req_fields)
        raw_res = requests.get(req_url, headers=self.headers)
        res_json = raw_res.json()
        if total_num == -1:
          total_num = res_json["nb_results"]
        # process results.
        res_items = res_json["files"]
        for item in res_items:
          try:
            cur_content_obj = self.convert_raw_content(item)
            yield cur_content_obj
          except Exception as ex:
            print("error converting adobe item: {}. raw item: {}".format(
                ex, item))
            continue
        # ready for next page.
        offset += len(res_items)
        if offset >= total_num:
          break
      except Exception as ex:
        print("error processing get_imgs: {}".format(ex))



"""Script for reddit api.
https://praw.readthedocs.io/en/latest/index.html
"""

import praw


class RedditAPI(object):
  """Class for reddit api.
  """

  def __init__(self, client_id, client_secret, user_agent):
    """Get credentials from here:
    https://github.com/reddit-archive/reddit/wiki/OAuth2-Quick-Start-Example#first-steps
    """
    self.client = praw.Reddit(
        client_id=client_id,
        client_secret=client_secret,
        user_agent=user_agent)
    print("reddit api initialized.")

  def get_subreddit_posts(self, subreddit_id, num=None):
    """Retrieve posts from a subreddit.

    Args:
      subreddit_id: id of a subreddit.
      num: number of posts, use None for all.
    """
    subreddit = self.client.subreddit(subreddit_id)
    posts = subreddit.hot(limit=num)
    all_posts = []
    for post in posts:
      post_obj = {
          "author": post.author,
          "url": post.url,
          "name": post.name,
          "title": post.title,
          "comments": [],
          "comment_num": post.num_comments
      }
      all_posts.append(post_obj)
    return all_posts



"""Common scraper class for movie data.
"""

import abc
import json
import os


class MovieObject(object):
  """Base class for a movie object.
  """
  # meta
  title = ""
  # description.
  desc = ""
  year = ""
  release_date = ""
  # list of internal genre names.
  genres = []
  # directors.
  directors = []
  # casts: [(last name, rest of the name)]
  casts = []
  # images.
  cover_img_url = ""
  backdrop_url = ""
  # sources: information from different sources.
  # [{name, rating, reviews}]
  source_data = []
  # trailer link.
  trailers = []
  # imdb id.
  imdb_id = ""
  # financials.
  budget = 0
  revenue = 0
  first_week_box = 0


class MovieAPIBase(object):
  """Base class for movie related api.
  """
  __metaclass__ = abc.ABCMeta
  internal_genres = []

  @abc.abstractmethod
  def __init__(self, api_key):
    """Set up class with api key. None for scraper.
    """
    # load genres.
    data_dir = os.path.join(os.path.dirname(__file__), "data")
    # with open(os.path.join(data_dir, "genre.json"), "r") as f:
    #   data = json.load(f)
    #   self.internal_genres = data["data"]
    print("init base movie api.")

  @abc.abstractmethod
  def get_movie_info(self, id):
    """Fetch movie information.

    Args:
      id: identifier for the movie, e.g. id, title.
    
    Returns:
      a movie object.
    """
    pass

  @abc.abstractclassmethod
  def search_movies(self, keywords):
    """Search movies with keywords.

    Returns:
      list of movie object.
    """
    pass

  @abc.abstractmethod
  def get_person_info(self, id):
    """Fetch info about a person.
    """
    pass

  def list_internal_genres(self):
    """A universally defined genres.
    """
    return self.internal_genres

  @abc.abstractmethod
  def map_to_internal_genre(self, genre):
    """Map a genre from a source to our internally defined one.

    Args:
      genre: name for the genre.

    Returns:
      internal genre name.
    """
    pass
